﻿/* Copyright (C) Click & Push Accessibility, Inc - All Rights Reserved
 * Unauthorized copying of this file, via any medium is strictly prohibited
 * Proprietary and confidential
 * Written and maintained by the Click & Push Development team 
 * <dev@clicknpush.ca>, January 2022
 */

import React from "react";
import { FontAwesome } from "@expo/vector-icons";
import { StyleSheet, Text, TouchableOpacity, View, ImageSourcePropType, SafeAreaView, FlatList } from "react-native";
import { Svg, Image as ImageSVG } from "react-native-svg";
import ViewShot, { captureRef } from "react-native-view-shot";
import { QueryStatus } from "react-query";
import { useAuth } from "../../../../data/Auth/AuthContext";
import { Landmark, useReportLandmark } from "../../../../data/landmarks";
import { UserProfile } from "../../../../data/profiles";
import { MainTabsNavigationProp } from "../../../../navigation/MainTabsNavigator";
import { lmTypes } from "../../../../utils/GlobalUtils";
import IndoorFloor from "../../IndoorFloor";
import { Report } from "./Report";
import TouchOpaq from './TouchOpaq';
// import { TextInput } from "react-native-paper";

interface DetailsHeaderProps {
    landmark?: Landmark,
    editingEnabled: boolean,
    toggleEditing: (state: boolean) => void,
    updatedLandmark?: Landmark,
    editLandmark: () => void,
    removeLandmark: () => void,
    toggleDetailsPanel: (state: boolean) => void,
    profile?: UserProfile,
    rateLandmark: (rating: 1 | -1) => void
    processingPhoto: boolean,
    addPhotoStatus: QueryStatus
    deletePhotoStatus: QueryStatus
    ratedByUser: boolean
    authNavigation: MainTabsNavigationProp
    toggleLmDetails: (state: boolean) => void
    place: string
    uri: String
}

/**
 * Component that renders the landmark details panel header
 * @param 
 */
export const DetailsHeader: React.FC<DetailsHeaderProps> = (props) => {
    const { landmarkOwnedByUser, anonUserId, setAlert } = useAuth()
    const photosAreBusy = () => {
        return props.processingPhoto ||
            props.addPhotoStatus == "loading" ||
            props.deletePhotoStatus == "loading"
    }

    const reportLandmarkMutation = useReportLandmark()
    const capture: any = React.createRef();
    const imgWidth = 346
    const imgHeight = 448
    const imageDim = 25

    /**
     * Calls the {@linkcode reportLandmark} mutation from the {@link useReportLandmark} hook.
     */
    const reportLandmark = async (landmark, reasons, extraComments) => {
        if (landmark.floor !== null) {  // Indoor landmark screenshot
            console.log('indoor landmark reported')
            const uri = await captureRef(capture, {
                format: "jpg",
                quality: 1,
                result: 'base64'
            })
            landmark['indoorLmLocImg'] = uri
        }
        else {  // Outdoor landmark screenshot
            console.log('outdoor landmark reported')
            landmark['indoorLmLocImg'] = props.uri
        }
        landmark['reasons'] = reasons
        landmark['isLandmark'] = true
        landmark['extraComments'] = extraComments

        await reportLandmarkMutation.mutateAsync(landmark);
    }

    const HeaderContent: React.FC = () => {
        // landmark is owned by user
        if (landmarkOwnedByUser(props.landmark) || (props.profile?.places == props.place)) {
            // editing is enabled
            if (props.editingEnabled) {
                return (
                    <View style={{ flexDirection: 'row', justifyContent: 'space-between', alignItems: "center", width: '100%' }}>
                        <TouchOpaq
                            func={() => props.toggleEditing(false)}
                            size={25}
                            col={"white"}
                            name={"close"}
                        />
                        {props.updatedLandmark?.description && props.updatedLandmark.landmark_type ? // only display check if fields are populated
                            // <FontAwesome style={{ marginTop: 2 }} size={25} color="white" name="check" onPress={async () => props.editLandmark()} /> 
                            <TouchOpaq
                                func={async () => props.editLandmark()}
                                size={25}
                                col="white"
                                name={"check"}
                            />
                            : null}
                    </View>
                )
            }
            // editing is disabled
            else {
                return (
                    <View style={{ borderColor: 'blue', borderWidth: 0, flexDirection: 'row', alignItems: "center", justifyContent: 'space-between', width: '100%' }}>
                        <View style={{ borderColor: 'green', borderWidth: 0, flexDirection: 'row', alignItems: "center", justifyContent: "flex-end" }}>
                            <TouchOpaq
                                func={() => props.toggleEditing(true)}
                                size={25}
                                col={"white"}
                                name={"edit"}
                            />
                            <TouchOpaq
                                func={() => props.removeLandmark()}
                                size={25}
                                col={"white"}
                                name={"trash"}
                            />
                            <Text style={{ color: 'white', fontSize: 20, marginTop: 2, marginLeft: 10 }} >{props.landmark?.rating}</Text>
                            <FontAwesome style={{ marginLeft: 5, marginBottom: 2, marginRight: 10 }} color="white" size={25} name="thumbs-up" />
                        </View>
                        <TouchOpaq
                            func={() => props.toggleDetailsPanel(false)}
                            size={25}
                            col={"white"}
                            name={"close"}
                        />
                    </View>
                )
            }
        }
        // landmark is not owned by user
        else {
            return (
                <View style={{ flexDirection: 'row', justifyContent: 'space-between', alignItems: "center", width: '100%' }}>
                    {props.ratedByUser ?  // landmark has already been liked by the current user
                        <View style={{ flexDirection: 'row' }} >
                            <Text style={{ color: 'white', fontSize: 20, marginTop: 2 }} >{props.landmark.rating}</Text>
                            <FontAwesome style={{ marginLeft: 5, marginTop: 2, marginRight: 30 }} color="white" size={25} name="thumbs-up" />
                            <Text style={{ color: 'lightgray', fontSize: 20, marginTop: 2, opacity: 0.7 }} >You liked this </Text>
                            <TouchableOpacity style={{ flexDirection: 'row' }} onPress={() => props.rateLandmark(-1)}>
                                <Text style={{ color: 'white', fontSize: 20, marginTop: 2, opacity: 0.8 }} >(</Text>
                                <Text style={{ color: 'white', fontSize: 20, marginTop: 2, opacity: 0.8, textDecorationLine: "underline" }} >Undo</Text>
                                <Text style={{ color: 'white', fontSize: 20, marginTop: 2, opacity: 0.8 }} >)</Text>
                            </TouchableOpacity>
                        </View> : // landmark has not been liked by user
                        <TouchableOpacity onPress={async () => { //this touchable will add a like to this landmark
                            if (anonUserId) {
                                setAlert({
                                    title: "Authentication required",
                                    type: "warning",
                                    message: "Oops! In order to rate landmarks, you have to log in.",
                                    callback: () => { props.toggleLmDetails(false); props.authNavigation.navigate("Account") },
                                    callbackButtonText: 'Go to login'
                                });
                            }
                            else if (!landmarkOwnedByUser(props.landmark)) {
                                await props.rateLandmark(1);
                            }
                        }}>
                            <View style={{ flexDirection: 'row' }} >
                                <Text style={{ color: 'white', fontSize: 20, marginTop: 2 }} >{props.landmark?.rating}</Text>
                                <FontAwesome style={{ marginLeft: 5, marginTop: 2 }} color="white" size={25} name="thumbs-up" />
                            </View>
                        </TouchableOpacity>}
                    <View style={{ flexDirection: 'row' }}>
                        {anonUserId ? null : <Report isLandmark={true} landmark={props.landmark} reportLandmark={reportLandmark} />}
                        <TouchOpaq
                            func={() => props.toggleDetailsPanel(false)}
                            col={"white"}
                            size={25}
                            name={"close"}
                        />
                    </View>
                    <ViewShot style={{ width: imgWidth + 20, height: imgHeight + 20, position: 'absolute', right: -2000 }} ref={capture} >
                        {props.landmark == null || props.landmark.floor == null || props.landmark.landmark_type == null ? <></> :
                            <View style={styles.container}>
                                <Svg>
                                    <IndoorFloor floorNum={props.landmark.floor} />
                                    <ImageSVG x={props.landmark.longitude * imgWidth - 3} y={props.landmark.latitude * imgHeight - 3} width={imageDim} height={imageDim} href={lmTypes[props.landmark.landmark_type]['image'] as ImageSourcePropType} />
                                </Svg>
                            </View>
                        }
                    </ViewShot>
                </View >
            )
        }
    }
    return (
        <View style={styles.detailsHeader}>
            {!photosAreBusy() ?
                <HeaderContent />
                : null}
        </View>
    )
}

const styles = StyleSheet.create({
    detailsHeader: {
        borderBottomWidth: 1,
        borderColor: 'white',
        flexDirection: 'row',
        justifyContent: 'space-between',
        alignItems: 'center',
        backgroundColor: '#df3f3f',
        marginBottom: 10,
        paddingVertical: 10,
        paddingHorizontal: 20
    },
    container: {
        aspectRatio: 8 / 10,  // (caters to portrait mode) (width is 80% the value of height dimension)
        width: '100%',
        height: '100%',
        maxWidth: "100%",
        maxHeight: "100%",
        backgroundColor: "white",
        padding: 10
    }
})